package jmunit.framework.cldc10;

import j2meunit.framework.Test;
import j2meunit.framework.TestCase;
import j2meunit.framework.TestSuite;

public class TimedMeasurementTest extends TestCase {

	// The OUTs.
	private TimedMeasurement timedMeasurement;
	private TimedMeasurement timedMeasurementWithTolerance;

	public void setUp() {
		this.timedMeasurement = new TimedMeasurement(100);
		this.timedMeasurementWithTolerance = new TimedMeasurement(100, 100);
	}

	public void testTimeOk() {
		this.timedMeasurement.startMeasurement();
		this.timedMeasurement.endMeasurement();
	}

	public void testTimeTooLong() throws InterruptedException {
		this.timedMeasurement.startMeasurement();
		Thread.sleep(150);
		try {
			this.timedMeasurement.endMeasurement();
			fail("Test took too long.");
		} catch (AssertionFailedException e) {

		}
	}

	public void testTimeToleranceOk() throws InterruptedException {
		this.timedMeasurementWithTolerance.startMeasurement();
		Thread.sleep(150);
		this.timedMeasurementWithTolerance.endMeasurement();
	}

	public void testConstructorStartsMeasurement() throws InterruptedException {
		Thread.sleep(150);
		try {
			this.timedMeasurement.endMeasurement();
			fail("Test took too long.");
		} catch (AssertionFailedException e) {

		}
	}

	public void testConstructorStartsMeasurementOk()
			throws InterruptedException {
		TimedMeasurement tm = new TimedMeasurement(1000);
		tm.endMeasurement();
	}
	
	// A clock resolution of zero should force the test to be delayed until the next clock tick.
	public void testTimeResolutionZero() {
		TimedMeasurement tm = new TimedMeasurement(100, 0);
		long time = System.currentTimeMillis();
		while (time == System.currentTimeMillis());
		time = System.currentTimeMillis();
		tm.startMeasurement();
		assertTrue(System.currentTimeMillis() != time);
	}

	public void runTest() throws Throwable {
		if (getName().equals("testTimeOk")) {
			testTimeOk();
		} else if (getName().equals("testTimeTooLong")) {
			testTimeTooLong();
		} else if (getName().equals("testTimeToleranceOk")) {
			testTimeToleranceOk();
		} else if (getName().equals("testConstructorStartsMeasurement")) {
			testConstructorStartsMeasurement();
		} else if (getName().equals("testConstructorStartsMeasurementOk")) {
			testConstructorStartsMeasurementOk();
		} else if (getName().equals("testTimeResolutionZero")) {
			testTimeResolutionZero();
		} else {
			fail("Unknown test: " + getName());
		}
	}

	public Test suite() {
		TestSuite suite = new TestSuite(this.getClass(),
				new String[] { "testTimeOk", "testTimeTooLong",
						"testTimeToleranceOk",
						"testConstructorStartsMeasurement",
						"testConstructorStartsMeasurementOk",
						"testTimeResolutionZero" });
		return suite;
	}
}
